;; Generated package description from autocrypt.el  -*- no-byte-compile: t -*-
(define-package "autocrypt" "0.4.2.0.20260126.200740" "Autocrypt implementation" '((emacs "24.3")) :commit "e65fc01502e2c02914752c5b58c100d47db40b30" :authors '(("Philip Kaludercic" . "philipk@posteo.net")) :maintainer '("Philip Kaludercic" . "philipk@posteo.net") :keywords '("comm") :url "https://codeberg.org/pkal/autocrypt.el")
